{% load i18n %}
<div class="xblock-header-secondary url-name-footer">
	<span class="url-name-label">{% trans "url_name for linking to this mentoring question set:" %}</span>
	<span class="url-name">{{ url_name }}</span>
</div>
