# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-19 13:57
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('problem_builder', '0005_auto_20170112_1021'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='answer',
            unique_together=set([('student_id', 'course_key', 'name')]),
        ),
        migrations.RemoveField(
            model_name='answer',
            name='course_id',
        ),
    ]
