{% load i18n %}
<div class="xblock-pb-slider">
  {% if not hide_header %}<h4 class="question-title">{{ title }}</h4>{% endif %}
  <div class="pb-slider-box clearfix">
    <p><label>{{ question|safe }} <span class="sr">({{instructions_string}})</span>
      <input type="range" id="{{ slider_id }}" class="pb-slider-range"
        min="0" max="100" step="1" value="{{initial_value}}"
      />
    </label></p>
    <div class="pb-slider-min-label" aria-hidden="true">{{ min_label }}</div>
    <div class="pb-slider-max-label" aria-hidden="true">{{ max_label }}</div>
  </div>
  <div class="clearfix">
    <span class="submit-result fa icon-2x checkmark-correct icon-ok fa-check"
      style="visibility: hidden;" aria-label="{% trans "Complete" %}"></span>
  </div>
</div>
