<table>
    {% if header_values %}
    <thead>
    {% for header in header_values %}
    <th>{{ header|safe }}</th>
    {% endfor %}
    </thead>
    {% endif %}
    <tbody>
    <tr>
        {% for content in content_values %}
        <td>
            <div class="mentoring-column">
                {{content|safe}}
            </div>
        </td>
        {% endfor %}
    </tr>
    </tbody>
</table>