/* Custom appearance for our "Add" buttons */
.xblock .add-xblock-component .new-component .new-component-type .add-xblock-component-button {
  width: auto;
  height: auto;
  line-height: 30px;
  padding-left: 1em;
  padding-right: 1em;
}

.xblock .add-xblock-component .new-component .new-component-type .add-xblock-component-button.disabled,
.xblock .add-xblock-component .new-component .new-component-type .add-xblock-component-button.disabled:hover {
  background-color: #ccc;
  border-color: #888;
  cursor: default;
}

.xblock[data-block-type=pb-mcq] .submission-message-help p,
.xblock[data-block-type=pb-mrq] .submission-message-help p,
.xblock[data-block-type=pb-rating] .submission-message-help p {
	border-top: 1px solid #ddd;
	font-size: 0.85em;
	font-style: italic;
	margin-top: 1em;
	padding-top: 0.3em;
}
