{% load i18n %}
{% if not hide_header %}
<h4 class="question-title" id="heading_{{ self.html_id }}">{{ self.display_name_with_default }}</h4>
{% endif %}
<fieldset class="choices questionnaire" id="{{ self.html_id }}"
  data-hide_results="{{ self.hide_results }}" data-hide_prev_answer="{{ hide_prev_answer }}">
  <legend class="question field-group-hd">{{ self.question|safe }}</legend>
  <div class="choices-list">
    {% for choice in custom_choices %}
    <div class="choice" aria-live="polite" aria-atomic="true">
      <button class="choice-result fa icon-2x"
           id="result_{{ self.html_id }}-{{ forloop.counter }}"
           aria-label=""
           data-label_correct="{% trans "Correct" %}"
           data-label_incorrect="{% trans "Incorrect" %}"></button>
      <label class="choice-label"
             for="{{ self.html_id }}_{{ forloop.counter }}"
             aria-describedby="feedback_{{ self.html_id }}
                                                    result_{{ self.html_id }}-{{ forloop.counter }}
                                                    choice_tips_{{ self.html_id }}-{{ forloop.counter }}">
        <span class="choice-selector">
          <input type="checkbox" name="{{ self.name }}" value="{{ choice.value }}"
             id="{{ self.html_id }}_{{ forloop.counter }}"
             {% if choice.value in self.student_choices and not hide_prev_answer %} checked{% endif %}
          />
        </span>
        <span class="choice-label-text">{{ choice.content|safe }}</span>
      </label>
      <div class="choice-tips-container">
        <div class="choice-tips" id="choice_tips_{{ self.html_id }}-{{ forloop.counter }}"></div>
      </div>
    </div>
    {% endfor %}
    <div class="feedback" id="feedback_{{ self.html_id }}"></div>
  </div>
</fieldset>
