{% load i18n %}
<div class="xblock-header-secondary url-name-footer">
  <span class="url-name-label">{% trans "ID for referencing this rating block:" %}</span>
  <span class="url-name">{{ url_name }}</span>
</div>
