# -*- coding: utf-8 -*-
# Generated by Django 1.11.25 on 2019-11-28 23:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('problem_builder', '0006_remove_deprecated_course_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='answer',
            name='student_id',
            field=models.CharField(db_index=True, max_length=50),
        ),
    ]
