.mentoring .answer.editable {
    height: 250px;
    width: 100%;
    margin-bottom: 10px;
}

.mentoring .xblock-answer .answer-checkmark {
    display: inline-block;
    margin-bottom: 20px;
    float: none;
}

.mentoring .answer.read_only {
    background: #f9f9f9;
    border-left: 10px solid #ccc;
    margin: 20px 10px;
    padding: 10px 10px;
    quotes: "\201C""\201D""\2018""\2019";
}

.mentoring .answer.read_only:before {
    color: #ccc;
    content: "\201C";
    font-size: 65px;
    line-height: 11px;
    margin-right: 13px;
    vertical-align: -25px;
}

.answer-table {
    margin-bottom: 20px;
}
