{% load i18n %}
<div class="mentoring themed-xblock" data-step="{{ self.step }}" data-feedback_label="{{ self.feedback_label }}" data-hide_feedback="{{ self.hide_feedback }}">
  <div class="missing-dependency warning" data-missing="{{ self.has_missing_dependency }}">
    {% with url=missing_dependency_url|safe %}
    {% blocktrans with link_start="<a href='"|add:url|add:"'>" link_end="</a>" %}
    You need to complete {{link_start}}the previous step{{link_end}} before
    attempting this step.
    {% endblocktrans %}
    {% endwith %}
  </div>

  {% if show_title and title %}
  <div class="title">
    <h3>{{ title }}</h3>
  </div>
  {% endif %}

  <div>

    {{child_content|safe}}

    {% if self.display_submit %}

    <div class="grade" data-score="{{ self.score.1 }}"
         data-correct_answer="{{ self.score.2|length }}"
         data-incorrect_answer="{{ self.score.3|length }}"
         data-partially_correct_answer="{{ self.score.4|length }}"
         data-max_attempts="{{ self.max_attempts }}"
         data-num_attempts="{{ self.num_attempts }}"
         data-extended_feedback="{%if self.extended_feedback %}True{% endif %}"
         data-assessment_review_tips="{{ self.review_tips_json }}"
         data-correct="{{ self.correct_json }}"
         data-incorrect="{{ self.incorrect_json }}"
         data-partial="{{ self.partial_json }}">
    </div>

    <div class="submit">

      <input type="button" class="input-main" value="{% trans "Submit" %}" disabled="disabled" />

      <div class="attempts" data-max_attempts="{{ self.max_attempts }}" data-num_attempts="{{ self.num_attempts }}"></div>
    </div>
    {% endif %}
    <div class="messages" tabindex="-1"></div>
    <div class="assessment-review-tips"></div>
  </div>
  <div class="review-link"><a href="#">{% trans "Review final grade" %}</a></div>

  <p class="copyright">
    Copyright &copy; 2013&ndash;2015 OpenCraft, Harvard, edX, McKinsey, and The People's Science, released under the
    <a target="_blank" href="https://github.com/open-craft/problem-builder/blob/master/LICENSE">{% trans "APGLv3 license" %}</a>
  </p>
</div>
