.mentoring .title .shared-header {
    margin: 1em 0;
}

.mentoring .messages {
    display: none;
}

.mentoring .messages .title1 {
    color: #333333;
    text-transform: uppercase;
    font-weight: bold;
    font-style: normal;
    font-size: 14px;
    margin-bottom: 5px;
}

.sr-only {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border: 0;
}

.mentoring .warning {
    border: 1px solid;
    margin: 10px 0;
    padding: 15px 10px;
    -moz-border-radius: 10px;
    -webkit-border-radius: 10px;
    border-radius: 10px;
    color: #9F6000;
    background-color: #FEEFB3;
}

.mentoring .missing-dependency.warning {
    display: none;
}

.mentoring .bold {
    font-weight: bold;
}
.mentoring .italic {
    font-style: italic;
}

.mentoring fieldset {
    margin-top: 10px;
}

.xblock .mentoring h4 {
    margin-top: 0;
    margin-bottom: 7px;
}

.mentoring .submit {
    display: inline-block;
    margin-top: 20px;
}

.mentoring .submit input {
    display: none;
}

.mentoring legend {
    white-space: normal;
    display: table; /* Enable line-wrapping in IE8 */
}

.mentoring .choices legend.question p:last-child { /* Selector must be more specific than 'div.course-wrapper section.course-content p' */
    margin-bottom: 0;
}

.mentoring .xblock-pb-slider p label {
    font-size: inherit;
}

.mentoring .pb-slider-box {
    max-width: 400px;
}

.mentoring .pb-slider-range {
    width: 100%;
}

.mentoring .pb-slider-min-label {
    float: left;
}

.mentoring .pb-slider-max-label {
    float: right;
}

.mentoring .clearfix::after {
    clear: both;
    display: block;
    content: " ";
}

.mentoring .attempts {
    margin-left: 10px;
    display: inline-block;
    vertical-align: middle;
    font-size: 13px;
    font-weight: 600;
}

.mentoring .attempts > span {
    color: #777;
    font-style: italic;
    -webkit-font-smoothing: antialiased;
}

.mentoring .checkmark-correct {
    font-size: 22pt;
    color: #629b2b;
    float: left;
}

.mentoring .checkmark-partially-correct {
    font-size: 22pt;
    color: #e37222;
    float: left;
}

.mentoring .checkmark-incorrect {
    font-size: 22pt;
    color: #c1373f;
    float: left;
}

.mentoring .choice-result,
.mentoring .choice-tips-container {
    vertical-align: top;
}

.mentoring .choice-label span.low {
    line-height: inherit;
}

.mentoring .choice .choice-tips-container.with-tips {
    border-left: 2px solid #ddd;
}

.mentoring .choice.correct .choice-tips-container.active {
    border-color: #629b2b;
}

.mentoring .choice.incorrect .choice-tips-container.active {
    border-color: #c1373f;
}

.mentoring .choice-result::before {
    content: "";
    display: block;
    width: 36px;
}

.mentoring .choice-result {
    display: none !important;
    height: 17px;
}

.mentoring .checkmark-correct,
.mentoring .checkmark-incorrect,
.mentoring .checkmark-partially-correct {
    display: inline-block !important;
    margin: -7px 0 0 -35px;
}

.mentoring .sb-review-score .checkmark-correct,
.mentoring .sb-review-score .checkmark-incorrect,
.mentoring .sb-review-score .checkmark-partially-correct,
.mentoring  .submit .checkmark-correct,
.mentoring  .submit .checkmark-incorrect,
.mentoring  .submit .checkmark-partially-correct{
    margin: 0;
}

.mentoring .checkmark-correct::before {
    content: "\f00c";
}

.mentoring .checkmark-incorrect::before {
    content: "\f00d";
}


.mentoring .assessment-question-block {
    border: 5px solid #e5ebee;
    padding: 20px;
}

.mentoring .assessment-checkmark {
    margin-right: 10px;
}

.mentoring .assessment-checkmark.checkmark-clickable {
    cursor: pointer;
}

.mentoring .grade .grade-result {
    margin: 20px;
}

.mentoring .grade .grade-result .results-section p {
    margin-bottom: 4px; /* Override LMS rule 'div.course-wrapper section.course-content p { margin-bottom: huge; }' */
}

.mentoring .grade .checkmark-incorrect {
    margin-left: 0;
    margin-right: 10px;
    width: 29px;
}

.mentoring input[type=button],
.mentoring input[type=button]:focus {
    background-color: #3384ca;
}

.mentoring input[type=button][disabled],
.mentoring input[type=button][disabled]:hover,
.mentoring input[type=button][disabled]:focus {
    background-color: #cccccc;
}

.mentoring input[type="checkbox"],
.mentoring input[type="radio"] {
    margin: 0;
}

.mentoring .review-list {
    list-style: none;
    padding-left: 0 !important;
    margin-top: 0;
    margin-left: 0;
    margin-bottom: 0.4em;
}
.mentoring .review-list li {
    display: inline;
}

.mentoring .review-list li a{
    font-weight: bold;
}

.mentoring .results-section {
    display: inline-block;
}

.mentoring .results-section p {
    margin-bottom: 4px;
    padding-top: 4px;
}

.mentoring .clear {
    display: block;
    clear: both;
}

.mentoring .review-link {
    float: right;
    display: none;
}

.mentoring p.review-tips-intro {
    margin-top: 1.2em;
    margin-bottom: 0;
    font-weight: bold;
}

.mentoring .review-tips-list {
    margin-top: 0;
    padding-top: 0;
}
.mentoring .review-tips-list li {
    margin-left: 0.5em;
    padding-left: 0;
}
.mentoring .review-tips-list li p {
    display: inline;
    margin: 0;
}


.pb-clarification span.clarification i {
    font-style: normal;
}

.pb-clarification span.clarification i:hover {
    color: rgb(0, 159, 230);
}

.mentoring .sb-step {
    position: relative;
}

.assessment-question-block div[data-block-type=sb-step],
.assessment-question-block div[data-block-type=sb-review-step] {
    display: none; /* Hidden until revealed by JS */
}

.mentoring .sb-step .sb-step-message {
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
    padding: 1.5em;
    background-color: white;
    box-shadow: 0 10px 20px #5C5C5C;
}

.mentoring .copyright {
    color: #AAA;
    clear: both;
    font-size: 10px;
    margin: 3em 0;
}

.mentoring .copyright a {
    color: #69C0E8;
}

div.course-wrapper section.course-content .mentoring p:empty {
    display: block;
    margin-bottom: 1.41575em;
}
