{% load i18n %}

<div class="add-xblock-component new-component-item adding">
  <div class="new-component">
    <ul class="new-component-type">
      <li><a href="#" class="single-template add-xblock-component-button" data-category="pb-column">{% trans "Add Answer Recap Column" %}</a></li>
    </ul>
  </div>
</div>
