.mentoring-table {
    background-size: 100% auto;
    background-repeat: no-repeat;
    background-position: center;
    margin-bottom: 20px;
}

.mentoring-table table {
    width: 100%;
    border: 1px solid rgb(221, 221, 221);
    border-spacing: 0;
    border-collapse: collapse;
}

.mentoring-table td,
.mentoring-table th {
    padding: 10px;
    vertical-align: top;
    border: 1px solid rgb(221, 221, 221);
}

.mentoring-table thead a {
    font-weight: bold;
}

.mentoring-table.immunity-map td {
    width: 25%;
}
.mentoring-table.immunity-map-assumptions td {
    width: 20%;
}
.mentoring-table.five-cols td {
    width: 20%;
}

.mentoring-table .mentoring-column {
    min-height: 500px;
}

.mentoring-table .hd-lvl1, .mentoring-table .hd-lvl2 {
    margin: 0 0 5px;
}

/* Hidden elements - Screenreaders */
.mentoring-table .cont-text-sr {
  border: 0;
  clip: rect(0 0 0 0);
  height: 1px;
  margin: -1px;
  overflow: hidden;
  padding: 0;
  position: absolute;
  width: 1px;
}

.mentoring-table-container .share-with-container {
    text-align: right;
}

.share-with-instructions {
    max-width: 14.5em;
    margin-bottom: 0;
    text-align: left;
}

.mentoring-share-panel {
    float: right;
    margin-bottom: .5em;
}

.mentoring-share-panel .mentoring-share-with {
    position: absolute;
    right: 3.15em;
    background-color: rgb(255, 255, 255);
    border: 2px solid rgb(221, 221, 221);
    padding: 1em;
    font-size: .8em;
}

.mentoring-share-with .share-header {
    text-align: left;
}

.mentoring-share-with .share-action-buttons {
    text-align: center;
    padding-top: .5em;
}

.mentoring-share-with .add-share-username {
    margin-right: 1em;
}

.mentoring-share-with .remove-share {
    color: black;
    margin-right: 1.45em;
}

.mentoring-share-with .add-share-field {
    line-height: normal;
    padding-top: .40em;
    padding-bottom: .40em;
}

.mentoring-share-with .share-errors {
    color: darkred;
    font-size: .75em;
    text-align: center;
    display: table-caption;
}

.new-share-container {
    margin-top: .5em;
    vertical-align: top;
    width: 100%;
    text-align: left;
}

ul.shared-list {
    padding-left: 0;
    padding-right: 0;
    margin: 0 0 .25em 0;
}

.share-errors-container {
    display: table;
    margin: 0 auto;
}

.shared-list li {
    list-style-type: none;
    display: block;
    padding: .25em 0 .25em 0;
    margin: 0;
}

.shared-list li .username {
    display: inline-block;
    float: left;
}

.share-panel-container {
    text-align: right;
}

.share-notification {
    border: 2px solid rgb(200, 200, 200);
    max-width: 15em;
    padding: 1em;
    background-color: rgb(255, 255, 255);
    position: absolute;
    right: 3.15em;
    font-size: .8em;
}

.share-notification .notification-close {
    float: right;
    font-size: 1.2em;
    color: black;
    cursor: pointer;
}

.report-download-container {
    text-align: right;
}

.mentoring .identification {
    padding-bottom: 1em;
}